@@ -1,4 +1,4 @@
-#include "tests.h"
+#include "wreport/tests.h"
 #include "bufr.h"
 
 using namespace wreport;
